@@ -1,16 +1,50 @@
 <?php
 
 function twc_weather_check_SM_version ($a = '0', $b = '0', $c = '0') {
+	if(!defined('SM_PATH'))
+	{
+        	define('SM_PATH','../../');
+		chdir ('..');
+        	include_once ('../src/validate.php');
+        	include_once ('../functions/page_header.php');
+        	include_once ('../src/load_prefs.php');
+        	include_once('../functions/i18n.php');
+		include_once('../functions/strings.php');
 
-       global $version;
-
-       list($aa, $bb, $cc) = preg_split('/\./', $version, 3);
-
-       if(!is_numeric($cc)) {
-           list($cc, $info) = explode(" ", $cc, 2);
-       }
-
-       return ($aa > $a)
+	}
+	else
+		{
+			include_once (SM_PATH . 'functions/strings.php');
+       	}
+	global $version;
+       	
+	list($aa, $bb, $cc) = preg_split('/\./', $version, 3);
+       	$tmp_cc = '';
+	
+       	if(!is_numeric($cc)) 
+       	{
+       		if (count (explode (" ", $cc, 2)) < 2)
+       		{
+       			for ($i=0; $i < strlen ($cc); $i++ )
+       			{
+       				if (is_numeric (substr($cc, $i, 1)))
+       				{
+       					$tmp_cc .= substr($cc, $i, 1);
+       				}
+       				else
+       				{
+       					$info = substr ($cc, $i);
+       					break;
+       				}
+       			}
+       			$cc = $tmp_cc;    				
+       		}
+       		else
+       		{
+           		list($tmp_cc, $info) = explode(" ", $cc, 2);
+        	}
+       	}
+       	return ($aa > $a)
            || (($aa == $a) && ($bb > $b))
            || (($aa == $a) && ($bb == $b) && ($cc >= $c));
 }
@@ -27,13 +61,17 @@ function SI_Temperature ($temp_F) {
 
 }
 
-function SI_Pressure ($Pressure_InchesHg) {
+function SI_Pressure ($Pressure_InchesHg, $pressure_units) {
 
 	$Pressure_MB = $Pressure_InchesHg * 33.8639;
+	$pressure = $Pressure_MB;
+	if ($pressure_units == 'kPa')
+	{
+   		$Pressure_kPa = $Pressure_MB * 0.1 ;
+   		$pressure = $Pressure_kPa;
+   	}
 
-   	$Pressure_KPa = $Pressure_MB * 0.1 ;
-
-        return (round($Pressure_KPa,1));
+        return (round($pressure,1));
 
 
 }
@@ -54,11 +92,15 @@ function SI_Length ($Distance_Inches) {
 
 }
 
-function SI_Speed ($Speed_Mph) {
+function SI_Speed ($Speed_Mph, $wind_units) {
 
 	$Speed_Kph = $Speed_Mph * 1.609;
-	
-	return (round ($Speed_Kph,0));
+	$Speed = $Speed_Kph;
+	if ($wind_units == "m/s")
+	{
+		$Speed = $Speed_Kph * 1000/3600;
+	}
+	return (round ($Speed,0));
 
 }
 
@@ -144,26 +186,26 @@ function fixBallParkTemps($temperature_phrase)
 }
 
 
-function fixWind($wind_speed_phrase)
+function fixWind($wind_speed_phrase, $wind_units)
 {
-	$wind_text = explode (" ",$wind_speed_phrase[0]);
+	$wind_text = explode (" ",$wind_speed_phrase);
 	foreach ($wind_text as $key => $value)
 	{
 		if (ereg("[0-9]+", $value))
 		{
-			$wind_text[$key]  = SI_Speed($value);
+			$wind_text[$key]  = SI_Speed($value, $wind_units);
 		}
 		elseif ( ereg("(mph)+", $value))
 		{
-			$wind_text[$key] = "km/h";
+			$wind_text[$key] = $wind_units;
 		}
 		if ( ereg("(\.)+", $value))
 		{
 			$wind_text[$key] .= ".";
 		}
 	}
-	$wind_speed_phrase[0] = implode (" ", $wind_text);
-	return ($wind_speed_phrase[0]);
+	$wind_speed_phrase = implode (" ", $wind_text);
+	return ($wind_speed_phrase);
 }
 
 /*
@@ -208,7 +250,7 @@ function fixEnglishNarrative ($narrative)
    degree symbol.  So trap them here for fixFahrenheit to fix up.
 */
                 $phrase = preg_replace_callback ("/\-?[0-9]+[F]{1}/", 
-				fixFahrenheit, $phrase);
+				'fixFahrenheit', $phrase);
 
                 $start_pos = $chop_pos + 1;
                 $new_narrative .= $phrase;
@@ -334,7 +376,7 @@ function fixInchesRange ($inches_phrase)
   modifiers like uppper, lower, mid, middle, etc. 
 
 */
-function SI_Narrative($narrative)
+function SI_Narrative($narrative, $wind_units)
 {
 	$start_pos = 0;
 	$chop_pos = 0;
@@ -367,30 +409,31 @@ function SI_Narrative($narrative)
                 $search = "/((three(\-|\ )+quarters)(\ of\ an inch)*".
                           "|one|two|three|four|five|six|seven|eight|nine|ten".
                           "|(quarter((\ of\ an\ inch)|(\ inch)))|half\ an\ inch|eighth|teens|single\ digits|an\ inch(\.*))/";
-                $phrase = preg_replace_callback ($search, fixWordstoNumbers, 
+                $phrase = preg_replace_callback ($search, 'fixWordstoNumbers', 
                           $phrase);
 		
 /* Find wind speed range:  XX to YY mph or just XX mph  */
 
 /* Sometimes the phrase will be XX-YY mph; replace the "-" with "to" */
 		$search = "/[0-9]+\-[0-9]+/";
-		$phrase = preg_replace_callback ($search, fixHyphen, $phrase);
+		$phrase = preg_replace_callback ($search, 'fixHyphen', $phrase);
  
 		$search = "/([0-9]+((\-)|(\ to\ {1}))[0-9]+\ {1}mph{1}\.*)".
-			  "|(to\ {1}[0-9]+\ {1}mph{1}\.*)|([0-9]+\ mph\.*)/";
-		$phrase = preg_replace_callback ($search, fixWind, $phrase);
+			  "|(to\ {1}[0-9]+\ {1}mph{1}\.*)|([0-9]+\ mph\.*)/e";
+			 
+		$phrase = preg_replace ($search, "fixWind('\\1', '$wind_units')", $phrase);
                 
 /* Find Temperature (Fahrenheit Variants  */
 
 /* Handle a range of temps from XX to YYF */
 
 		$search = "/[0-9]+\ to\ {1}\-?[0-9]+F{1}\.*/";
-		$phrase = preg_replace_callback ($search, fixTemperatureRange, 
+		$phrase = preg_replace_callback ($search, 'fixTemperatureRange', 
 			  $phrase);
 	
 /* Handle a single temp:  XXF  */
 
-		$phrase = preg_replace_callback ("/\-?[0-9]+[F]{1}/", fixTemps, 
+		$phrase = preg_replace_callback ("/\-?[0-9]+[F]{1}/", 'fixTemps', 
  			  $phrase);
 
 /* TWC likes to use words for numerical values when expressing temperature 
@@ -399,7 +442,7 @@ function SI_Narrative($narrative)
 
 		$search = "/((in\ the\ )*(|high|low|upper". 
 			  "|lower|middle|mid)*\ )\-*[0-9]+[s]{1}/";
-		$phrase = preg_replace_callback ($search, fixBallParkTemps, 
+		$phrase = preg_replace_callback ($search, 'fixBallParkTemps', 
 			  $phrase);
 	
 /* Convert Precipitaion (Inches variants) */
@@ -409,7 +452,7 @@ function SI_Narrative($narrative)
 		$search = "/((three(\-|\ )+quarters)(\ of\ an)*".
 			  "|one|two|three|four|five|six|seven|eight|nine|ten".
 			  "|quarter|half|eighth|an\ inch(\.*))/";
-		$phrase = preg_replace_callback ($search, fixWordstoNumbers, 
+		$phrase = preg_replace_callback ($search, 'fixWordstoNumbers', 
 			  $phrase);
 */
 /* Handle ranges of precipitation: to XX inch(es) or XX to YY inch(es) */
@@ -417,14 +460,13 @@ function SI_Narrative($narrative)
 		$search = "/(to\ ([0-9](\.[0-9])+)*\ (inches|inch)\.*)".
 			  "|([0-9]+(\.[0-9]+)*\ to\ {1}[0-9]+(\.+[0-9]+)*".
 			   "\ (inches|inch)+\.*)/";
-		$phrase = preg_replace_callback ($search, fixInchesRange, 
-			  $phrase);
+		$phrase = preg_replace_callback ($search, 'fixInchesRange',$phrase);
 
 /* Handle single instance of precip:  an XX inch(es) */
 
 		$search = "/(!(than)&an\ |a\ )*([0-9]+(\.[0-9]+)*\ )*(inches|inch){1}\.*/";
 
-		$phrase = preg_replace_callback ($search, fixInches, $phrase);
+		$phrase = preg_replace_callback ($search, 'fixInches', $phrase);
 /* 
    Almost done, remove any extra spaces that were introduced during conversion
 */
@@ -438,5 +480,170 @@ function SI_Narrative($narrative)
 	return ($new_narrative);
 
 }
+function twc_display_wicon($tmp_URL) 
+{	
 
+	/* take in a URL pointing to a GIF and spit it out to a client as PNG.
+	   Guess I'd call that a twc_weather image proxy (sans cache)?  The
+	   idea here is to allow the SM server to go out and gather the weather
+	   icons and feed them to the SM clients rather than redirecting the
+	   clients to get them themselves.  This should eliminate the annoying
+	   pop-up warning users get when they are accessing SM via a secure 
+	   (HTTPS) connection and the icon's URL is somewhere's else (like at 
+	   twc's site) and can only be gotten via a clear channel HTTP
+	   connection.
+	 */
+
+	global $weather_proxy_icons_method;
+	global $weather_proxy, $weather_datafeed_timeout, $debug;
+
+	if(!defined('SM_PATH'))
+	{
+        	define('SM_PATH','../../');
+	}
+
+	if (twc_weather_check_SM_version (1, 3, 2))
+	{
+		if (file_exists (SM_PATH . 'plugins/twc_weather/config.php'))
+			include_once (SM_PATH . 'plugins/twc_weather/config.php');
+		else
+			include_once (SM_PATH . 'plugins/twc_weather/config.php.sample');
+	}
+	else
+	{
+        	if (file_exists ('config.php'))
+                	include_once ('config.php');
+        	else
+                	include_once ('config.php.sample');
+	}
+
+	if (isset ($_GET['image-passthru']))
+	{
+		$weather_image_passthru = TRUE;
+		$tmp_URL = $_GET['image-passthru'];
+	}
+	else
+	{
+		$weather_image_passthru = FALSE;
+	}
+	$request = 'GET ';
+	
+	if (preg_match ("/^(http:\/\/)?([^\/]+)([a-z0-9\/\.\-\*\^\$\&\#\@\%\_\=\+]*)/i", $tmp_URL, $matches))
+	{
+		$the_server = $matches[2];
+		$img_server = $the_server;
+		$tmp_URI = $matches[3];
+		$request .= $tmp_URL ;
+	}
+	else 
+	{
+		die ("Internal Error opening WICON GIF. " .
+			"URL likely not found!\r\nGot URL: $tmp_url\r\nProxy: $weather_proxy\r\n");
+	}
+	
+	if ($weather_proxy)
+        {
+                        preg_match ("/^(http:\/\/)?([^\/]+):(\d+)/i", $weather_proxy, $matches);
+
+                        $the_server = $matches[2];
+                        $the_port   = $matches[3];
+              
+ 	}
+        else
+        {
+                        $the_port = 80;
+        }
+
+          	$request .= " HTTP/1.1\r\n";;
+        
+                $request .= "Host: $img_server\r\n";
+                $request .= "Accept: */*\r\n";
+                $request .= "Connection: close\r\n";
+                $request .= "\r\n\r\n";
+		$data = '';
+		
+		/* Make a call to TWC -- only wait around $weather_datafeed_timeout seconds or bail out */                
+                $httpsock = @fsockopen ($the_server, $the_port, $errno, $errstr, $weather_datafeed_timeout);
+                if ($httpsock)
+                {
+			@stream_set_timeout ($httpsock, $weather_datafeed_timeout);
+                	@fwrite ($httpsock, $request);
+
+			$timer = time();
+        		while ( !@feof($httpsock) && (time() < $timer + $weather_datafeed_timeout))
+        		{
+        			$data .= @fread ($httpsock, 8192);
+        		}
+			@fclose($httpsock);
+        		
+		}
+		
+		$image = substr($data, strpos($data,"\r\n\r\n")+4);
+
+	if ($debug)
+	{
+		global $weather_debug_dir;
+		$fp = fopen ($weather_debug_dir . '/twc_weather-image.log', 'a');
+		fwrite ($fp, "URL: $tmp_URL\n");
+		fwrite ($fp, "URI: $tmp_URI\n");
+		fwrite ($fp, "Server: $the_server\n");
+		fwrite ($fp, "Image Server: $img_server\n");
+		fwrite ($fp, "Port: $the_port\n");
+		fwrite ($fp, "Timeout: $weather_datafeed_timeout\n");
+		fwrite ($fp, "Request: $request\n");
+		if (isset ($_GET['image-passthru']))
+			fwrite ($fp, "Passthru: " . $_GET['image-passthru']. "\n");
+		fwrite ($fp, "$image");
+		fwrite ($fp, "\n");
+		fclose ($fp);
+	}
+	
+if ($weather_proxy_icons_method == "TRANSPARENT" || $weather_image_passthru)
+{
+	header("Content-type: image/gif");
+	header("Content-length: " & strlen($image));
+	header("Cache-Control: public");
+        header("X-Image-ID: $tmp_URL");
+	
+		echo "$image";
+}
+
+else	
+{
+	/* Transcoding Proxy functionality (PNG by hardcoded by default) */	
+
+	header("X-Image-ID: $tmp_URL");
+
+        /*
+           Unhack the name that we hacked so the client browser would
+           name the icon correctly in the HTML source.
+        */
+	if (preg_match ("/png/", $tmp_URL))
+	{
+		$tmp_URL = preg_replace("/png/", 'gif', $tmp_URL);
+	}
+
+	/* Assume original URL to wicon is a GIF */
+	
+	$img_in = imagecreatefromgif("https://" . $_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'] . '?wicon='. $tmp_URL . "&image-passthru=$tmp_URL");
+
+        $in_x = imagesx($img_in);
+        $in_y = imagesy($img_in);
+
+	/* Spit out a PNG header type */
+	header("Content-type: image/png");
+
+        /* not sure if determining length this way is a valid assumption */
+	header("Content-length: " . ($in_x * $in_y));
+	
+	/* In case anybody (proxy) cares to save some bandwidth */
+	header("Cache-Control: public");
+
+	/* All done, spit out PNG instead of GIF to the client */	
+        imagepng($img_in);
+}
+
+return (TRUE);
+
+}
 ?>
