@@ -3,6 +3,109 @@
 /*******************************************************************************
 
 $Log: weather.php,v $
+Revision 1.52  2005/03/13 09:28:06  jamuel
+Minor update to the datafeed URL
+
+Revision 1.51  2005/02/15 23:42:33  jamuel
+Minor fix to the location parsing routne to look for "Right Now for"
+instead of "Travel Forecast for".
+
+Revision 1.50  2005/02/14 23:44:21  jamuel
+Fixed the parser to handle changed in datafeed that was breaking the
+wicon and current temperature.
+
+Revision 1.49  2005/01/20 04:19:29  jamuel
+Minor fix to datafeed parser that was not correctly handling the scraping of the city name which corrupted userprefs for SM installations that utilize file-based userprefs.
+
+Revision 1.48  2005/01/15 04:32:42  jamuel
+Separate commit to update how the parser deals with the barometric pressure
+rising and falling icon (new in the datafeed).  Adds weather_proxy_icon
+support.
+
+Revision 1.47  2005/01/15 03:21:44  jamuel
+Updated the datafeed parser logic to support new datafeed format.
+
+Revision 1.46  2004/11/24 20:02:56  jamuel
+Rework for proxy server fixes and for socket timeout fixes.
+
+Revision 1.45  2004/10/07 18:01:47  jamuel
+Face lift to handle plugin architectural guidelines, added a
+twc_weather-version() function, bumped the version, and README.
+
+Revision 1.44  2004/10/07 05:43:01  jamuel
+Fixes for weather_proxy support in options.php, switched out requires
+with includes, fixed narratives parser (wasn't catching text with %
+sign).
+
+Revision 1.43  2004/10/02 06:00:18  jamuel
+Updates to support user selectable barometric pressure SI units as hPa
+or kPa.
+
+Also fixed a nagging bug in the options screen that caused non-
+US locations to be tagged not found because the weather code was not being
+used.  The new functionality is if the Location doesn't change (i.e. some
+other attributes are changed) the script doesn't attempt to validate the
+location (since it ought to still be good shouldn't it).
+
+Revision 1.42  2004/09/29 00:39:14  jamuel
+Minor updates to I18n related code.  Also bumped version and README
+in preparation for 1.3 beta 3 release.
+
+Revision 1.41  2004/09/28 05:09:19  jamuel
+Left $debug flag on.
+
+Revision 1.40  2004/09/28 04:59:24  jamuel
+Reworked all string outputs to support I18n
+
+Revision 1.39  2004/09/27 16:52:06  jamuel
+Major overhaul to eliminate most all (I hope) E_ALL info messages.
+
+Revision 1.37  2004/09/26 07:05:53  jamuel
+Added the I18n hooks for translations.
+
+Revision 1.35  2004/09/22 20:51:00  jamuel
+Multiple commits to get up to 1.2 RC2a with a fix to options.php that was
+causing blank pages when clicking Hide or Show on locations.
+
+Revision 1.48  2003/09/23 00:01:50  jamuel
+Mostly hacks to twc_support to enable transparent wicon proxying.  Also
+worked on weather.php to handle mangling of the wicon URL correctly
+so that a PNG is called a PNG and a GIF called a GIF when/where appropriate.
+
+Config has a new setting to deal with the transparent proxying.
+
+Revision 1.47  2003/07/24 16:13:35  jamuel
+
+Added additional debugging information to capture files.
+
+Revision 1.46  2003/07/15 18:12:12  jamuel
+
+Added quick and dirty cache validation scheme to load_locations().  Check
+the temperature that's retrieved from the cache, if contains a numerical
+value then the cache *probably* has good data.  Set's cache_valid to TRUE
+then when weather_display is called a cache_valid is added to the cache
+checking conditional.
+
+Revision 1.45  2003/07/10 18:39:43  jamuel
+
+Made WIcon Proxy on by default, fixed extended forecasts to work beyhond
+5 days, changed weather.com URL back to www from w3, added @ error
+suppression around some of the File/Socket IO.
+
+Revision 1.44  2003/06/16 17:16:22  jamuel
+
+Minor formatting change to center the location and/or alias when no wicon
+is displayed.
+
+Revision 1.43  2003/06/05 16:48:26  jamuel
+Turned off debugging\!
+
+Revision 1.42  2003/06/05 16:38:50  jamuel
+
+Added support for forcing the SM server to proxy client request for the
+weather icons rather than having the client redirected to the datafeed
+for the icons.
+
 Revision 1.41  2003/05/26 17:58:25  jamuel
 
 Major changes to bring in 1.2 features (weather location templates) and
@@ -260,10 +363,6 @@ Added CVS Keywords ($) to source file.
 
 global $debug; 
 
-require_once ('twc_support.php');
-
-$debug = FALSE;
-
 /* set this to true if you want to skip caching during debugging */
 
 /* SM Version Checking */
@@ -274,11 +373,24 @@ if (twc_weather_check_SM_version (1, 3, 2))
 	{	
 		define('SM_PATH','../../');
 	}
-	require_once (SM_PATH . 'plugins/twc_weather/chunky.php');
+	include_once (SM_PATH . 'plugins/twc_weather/chunky.php');
+	include_once (SM_PATH . 'plugins/twc_weather/twc_support.php');
+
+	if (file_exists (SM_PATH . 'plugins/twc_weather/config.php'))
+		include_once (SM_PATH . 'plugins/twc_weather/config.php');
+	else
+		include_once (SM_PATH . 'plugins/twc_weather/config.php.sample');
+	
 }
 else
 {
-        require_once ('chunky.php');
+        include_once ('chunky.php');
+	include_once ('twc_support.php');
+
+       	if (file_exists ('config.php'))
+               	include_once ('config.php');
+       	else
+               	include_once ('config.php.sample');
 }
 
 function load_locations ()
@@ -337,7 +449,17 @@ function load_locations ()
 		$locations[$i]['narrative-days'] = $temp[41];
 		$locations[$i]['use-metric'] = $temp[42];
 		$locations[$i]['forecast-narrow'] = $temp[43];
-	}
+		if (preg_match("/^[0-9]/", $locations[$i]['cache-temp']))
+		{
+			$locations[$i]['cache_valid'] = TRUE;
+		}
+		else
+		{
+			$locations[$i]['cache_valid'] = FALSE;
+		}
+		$locations[$i]['wind-units'] = $temp[44];
+		$locations[$i]['pressure-units'] = $temp[45];
+	}	
 
 	return $locations;
 }
@@ -356,158 +478,183 @@ function weather_narrative($location, $theid)
 {
 	global $color, $left_size, $data_dir, $username, $weather_proxy; 
 	global $weather_datafeed_timeout, $debug;
-
+	$request = 'GET ';
+	$the_server = 'www.weather.com';
 	if ($weather_proxy)
         {
                         preg_match ("/^(http:\/\/)?([^\/]+):(\d+)/i", $weather_proxy, $matches);
 
+			$request .= "http://$the_server";
                         $the_server = $matches[2];
                         $the_port   = $matches[3];
  	}
         else
         {
-                        $the_server = "w3.weather.com";
                         $the_port = 80;
         }
 
-         if (substr($location['place'], 0,2) == "US" )
-         {
-          	$request  = "GET /weather/narrative/" . $location['code'] . "?whatprefs=&GO=GO HTTP/1.1\r\n";
-         }
- 	else
-	{ 
- 		$request  = "GET /outlook/travel/narrative/" . $location['code'] . " HTTP/1.1\r\n";
- 	}
-                $request .= "Host: w3.weather.com\r\n";
+          	$request .= "/weather/mpdwcr/narrative?locid=" . $location['code'] . 
+			    "&adprodname=lap_travel_daily_main&channel=businesstraveler" . " HTTP/1.1\r\n";;
+        
+                $request .= "Host: www.weather.com\r\n";
                 $request .= "Accept: */*\r\n";
                 $request .= "Connection: close\r\n";
                 $request .= "\r\n\r\n";
-
-                $httpsock = fsockopen ($the_server, $the_port);
-                fwrite ($httpsock, $request);
-                $data = "";
-	/* Make a call to TWC -- only wait around $weather_datafeed_timeout seconds or bail out */
-	$timer = time();
-        while ( !feof($httpsock) && (time() < $timer + $weather_datafeed_timeout))
-        {
-        	$data .= fread ($httpsock, 8192);
-        }
-
-        $data = Decode_Chunked_Message ($data);
-	
-	$temp_narrative = strstr($data, "Detailed Local Forecast for");
-	$temp_narrative = strstr($temp_narrative, "<!-- begin loop -->");
-	
-	switch ($location['narrative-days'])
-	{
-		case 1:
-		    	$i = 2;
-		    	break;
-		case 2:
-			$i = 4;
-			break;
-		case 3:
-			$i = 5;
-			break;
-		case 4:
-			$i = 6;
-			break;
-		case 5:
-			$i = 7;
-			break;
-		case 6:
-			$i = 8;
-		case 7:
-			$i = 9;
-		default:
-			if ($location['view-narrative'])
-			{
-				$i = 3;
-			}
-			else
-			{	
-				$i = $location['narrative-days'];
-			}
-	}
-	$narrative_count = $i;
-	for ($i = 1; $i <= $narrative_count; $i++)
-	{
-		$temp_narrative = strstr($temp_narrative, "<!-- insert day/date/time of day -->");
-		$new_narrative  = substr($temp_narrative, 36, strpos($temp_narrative, "</TD") - 46);
-		$narrative .= '<B>'.$new_narrative.'<BR>'; 
-		$temp_narrative = strstr($temp_narrative, $new_narrative);
-	}
+		$data = "";
 		
-	return $narrative;
+		/* Make a call to TWC -- only wait around $weather_datafeed_timeout seconds or bail out */                
+                $httpsock = @fsockopen ($the_server, $the_port, $errno, $errstr, $weather_datafeed_timeout);
+                if ($httpsock)
+                {
+			@stream_set_timeout ($httpsock, $weather_datafeed_timeout);
+                	@fwrite ($httpsock, $request);
+
+			$timer = time();
+        		while ( !@feof($httpsock) && (time() < $timer + $weather_datafeed_timeout))
+        		{
+        			$data .= @fread ($httpsock, 8192);
+        		}
+			@fclose($httpsock);
+			if (preg_match ('/Transfer\-Encoding\:\ chunked/', $data))
+			{
+				$data = Decode_Chunked_Message ($data);
+			}
+		}
+		
+	$narrative_days = $location['narrative-days'];
+	$narrative = '';
+	if ($cntMatches = preg_match_all ('/day\[\'([0-9]+)\'\]\=new\ mpdNDObj\(new\ Date\([\'0-9\,]*\)\,\'([0-9]+)\'\,\'([a-z0-9\s\,\.\+\-\%]*)\'\,\'([a-z0-9\s]*)\'\)\;/i', $data, $all_matches))
+	{
+		for ($i = 0; $i < 11; $i++)
+		{
+			if ($all_matches[2][$i] <= $narrative_days)
+			{	
+				$narrative .= '<b>' ;
+				if (!empty ($all_matches[4][$i]))
+				{ 
+					$narrative .= _($all_matches[4][$i]) . '</b> '. $all_matches[3][$i] . '<br>';
+				}
+			}
+		}			
+	}
+	
+	$narrative = preg_replace('/[\n]/', '', $narrative);
+	
+	if ($debug)
+	{
+		global $weather_debug_dir;
+		$fp = fopen($weather_debug_dir . "/twc_weather-narrative.log", "w");
+		fwrite ($fp, "Server: $the_server\n");
+		fwrite ($fp, "Port: $the_port\n");
+		fwrite ($fp, "Request: $request\n");
+		fwrite ($fp, "Raw Data: $data\n");
+		fwrite ($fp, "Narrative: $narrative\n");
+		fclose ($fp);
+	}
+	return ($narrative);
 }
 function weather_display ($location, $theid)
 {
 	global $color, $left_size, $data_dir, $username, $weather_proxy,
-	       $weather_datafeed_timeout, $weather_cache_time_minimum, $debug;
+	       $weather_proxy_icons, $weather_proxy_icons_method,
+	       $weather_datafeed_timeout, $weather_cache_time_minimum, 
+	       $weather_location_amount, $debug;
 	
 	$cache_time = $weather_cache_time_minimum;
 	$thetime = time (NULL);
 	$cachetime = $location['timestamp'] + $cache_time;
-
-	if (($cachetime < $thetime) || $debug===TRUE)
+	
+	if (($cachetime < $thetime) || $debug === TRUE || 
+	     $location['cache_valid'] === FALSE)
 	{
+		$the_server = 'www.weather.com';
+		$request = 'GET ';
 		if ($weather_proxy)
 		{
-			preg_match ("/^(http:\/\/)?([^\/]+):(\d+)/i", $weather_proxy, $matches);
-
+			preg_match ("/^(http:\/\/)?([^\/]+):(\d+)/i", 
+			            $weather_proxy, $matches);
+			
+			$request .= "http://$the_server";
 			$the_server = $matches[2];
 			$the_port   = $matches[3];
+			
 		}
 		else
 		{
-			$the_server = "w3.weather.com";
+			$the_server = "www.weather.com";
 			$the_port = 80;
 		}
 
 		if (substr($location['code'], 0,2) == "US" )
 		{ 
-                	$request  = "GET /weather/local/" . $location['code'] . "?whatprefs=&GO=GO HTTP/1.1\r\n";
+                	$request  .= "/outlook/travel/businesstraveler/local/" . $location['code'] . 
+			            " HTTP/1.1\r\n";
                 }
 		else
 		{
-		$request  = "GET /outlook/travel/local/" . $location['code'] . " HTTP/1.1\r\n";
+		$request  .= "/outlook/travel/businesstraveler/local/" . $location['code'] . 
+			     " HTTP/1.1\r\n";
 		}
-		$request .= "Host: w3.weather.com\r\n";
+		$request .= "Host: www.weather.com\r\n";
 		$request .= "Accept: */*\r\n";
 		$request .= "Connection: close\r\n";
 		$request .= "\r\n\r\n";
-
-		$httpsock = fsockopen ($the_server, $the_port);
-		fwrite ($httpsock, $request);
-		$data = "";
-
-		/* 
-		   Make a call to TWC -- only wait around 
-		   $weather_datafeed_timeout seconds or bail out 
-		*/	
-
-		$timer = time();
-
-		while ( !feof($httpsock) && (time() < $timer + $weather_datafeed_timeout))
+		
+		$data = '';
+		$httpsock = @fsockopen ($the_server, $the_port, $errno, $errstr, $weather_datafeed_timeout);
+		if ($httpsock)
 		{
-			$data .= fread ($httpsock, 8192);
+			@stream_set_timeout ($httpsock, $weather_datafeed_timeout);
+			@fwrite ($httpsock, $request);
+			
+
+			/* 
+		   	Make a call to TWC -- only wait around 
+		   	$weather_datafeed_timeout seconds or bail out 
+			*/	
+
+			$timer = time();
+
+			while ( !@feof($httpsock) && (time() < $timer + $weather_datafeed_timeout))
+			{
+				$data .= @fread ($httpsock, 8192);
+			}
+			@fclose($httpsock);
+			if (isset ($debug) && $debug)
+			{	
+				$prechunked_data = $data;
+			}
+			if (preg_match ('/Transfer\-Encoding\:\ chunked/', $data))
+                        {
+  				$data = Decode_Chunked_Message ($data);
+			}
 		}
-		
-  		$data = Decode_Chunked_Message ($data);
-
-		$temp1 = strstr ($data, "Local Forecast for");
+		$temp1 = strstr ($data, "Right Now for");
+		$cut_at = strlen ('Right Now for</B><BR>');
 		if (strlen ($temp1) == 0)
+		{
 			$temp1 = strstr ($data, "Travel Forecast for");
-
-		$temp0 = strstr ($temp1, "<!-- http://image.weather.com");
-		$temp0 = strstr ($temp1, "obsInfo1");
-		$temp0 = strstr ($temp0, "<IMG SRC=");
-                $icon = substr ($temp0, 0 , strpos ($temp0, "WIDTH")) . '>';
- 
-                $citytitle = substr ($temp1, 19, strpos ($temp1, "</B>") - 19);
+			$cut_at = strlen("Travel Forecast for");
+		}
+		$temp0 = strstr ($data, "<IMG SRC=http://image.weather.com");
+		if ($weather_proxy_icons == TRUE)
+		{
+			$icon = substr ($temp0, 9 , strpos ($temp0, "WIDTH")) . '>';
+			if ($weather_proxy_icons_method != "TRANSPARENT")
+			{
+				/* Need this little hack to get the image named
+				   correctly on the client browswer */
+				$icon = preg_replace("/gif/", "png", $icon);
+			}
+			$icon = '<IMG SRC=' . SM_PATH . 'plugins/twc_weather/twc_image_handler.php?wicon=' . $icon;
+		}
+                else
+		{
+			$icon = substr ($temp0, 0 , strpos ($temp0, "WIDTH")) . '>';
+ 		}
+                $citytitle = ltrim(substr ($temp1, $cut_at, strpos ($temp1, "<BR><FONT") - $cut_at));
 		
-		$temp4 = strstr ($temp0, "obsInfo2");
-		$temp4 = strstr ($temp4, "obsTempTextA");
+		$temp4 = strstr ($temp0, "obsTempTextA");
 		if ($location['use-metric'])
 		{
 			$temperature = SI_Temperature (substr ($temp4, 13, strpos ($temp4, "&") - 13)) . "&deg;C";
@@ -516,42 +663,41 @@ function weather_display ($location, $theid)
 		{
 			$temperature = substr ($temp4, 13, strpos ($temp4, "&") - 13) . "&deg;F";
 		}
-                $temp3 = strstr ($temp4, "obsTextA");
+                $temp3 = strstr ($temp0, "obsTextA");
                 $conditions = substr ($temp3, 9, strpos ($temp3, "</B") - 9);
 
 		$temp5 = strstr ($data, "Feels Like");
-		
-		if ( preg_match ("/(N\/A)+/", $temp5) )
+		if ( preg_match ("/(N\/A)+/", substr($temp5, 1, 50), $matches) )
 		{
 			$feelslike = $temperature;
+
 		}
 		else
 		{
 			if ($location['use-metric']) 
                         {
-				$feelslike = SI_Temperature( substr ($temp5, 14, strpos ($temp5, "&") - 14)) . "&deg;C";
+				$feelslike = SI_Temperature( substr ($temp5, 15, strpos ($temp5, "&deg") - 15)) . "&deg;C";
 			}
 			else
 			{
-				$feelslike = substr ($temp5, 14, strpos ($temp5, "&") - 14) . "&deg;F";
+				$feelslike = substr ($temp5, 15, strpos ($temp5, "&deg") - 15) . "&deg;F";
 			}
 		}
-		$temp6 = strstr ($data, "UV Index:");
-		$temp6 = strstr ($temp6, "obsInfo2");
-		$uv = substr ($temp6, 9, strpos ($temp6, "</TD>") - 9);
+		$temp6 = strstr ($data, "uv0");
+		$uv = substr ($temp6, 4, strpos ($temp6, "&") - 4);
 
 		$temp7 = strstr ($data, "Wind:");
-		$temp7 = strstr ($temp7, "obsInfo2");
-		$wind = substr ($temp7, 9, strpos ($temp7, "</TD>") - 9);
-		if (strlen (strstr ($wind, "calm")) > 0) 
+		$temp7 = strstr ($temp7, "obsTextA");
+		$wind = substr ($temp7, 10, strpos ($temp7, "</td>") - 10);
+		if (strlen (strstr ($wind, "CALM")) > 0) 
 		{
 			$wind = "Calm";
 		}
 		else     
 		{
-			if (strlen (strstr($wind,"From the")) > 0)	
+			if (strlen (strstr($wind,"From")) > 0)	
 			{
-				$wind = substr ($wind, 9);
+				$wind = substr ($wind, 4);
 			}
 			else
 			{
@@ -574,10 +720,10 @@ function weather_display ($location, $theid)
 			$wind = substr ($wind , $pos_at , $pos_mph - $pos_at);
 			if ($location['use-metric'])
 			{
-				$wind = SI_Speed($wind);
+				$wind = SI_Speed($wind, $location['wind-units']);
 				if (strlen ($temp_gusting) == 0)
 				{
-					$wind .= " km/h";
+					$wind .= " " . $location['wind-units'];
 				}
 			}
 			else
@@ -588,14 +734,14 @@ function weather_display ($location, $theid)
 
 			if ($location['use-metric'] && strlen($temp_gusting) > 0)
 			{
-				$wind .= " gusting to ". SI_Speed($temp_gusting) . " km/h";
+				$wind .= " gusting to ". SI_Speed($temp_gusting, $location['wind-units']) . " ". $location['wind-units'];
 			}
 		}
 
 		
 		$temp8 = strstr ($data, "Dew Point:");
-		$temp8 = strstr ($temp8, "obsInfo2");
-		$dew = substr ($temp8, 9, strpos ($temp8, "&") - 9);
+		$temp8 = strstr ($temp8, "obsTextA");
+		$dew = substr ($temp8, 10, strpos ($temp8, "&") - 10);
 		if ($location['use-metric'] == 1)
 		{	
 			$dew = SI_Temperature ($dew) . '&deg;C';
@@ -606,19 +752,22 @@ function weather_display ($location, $theid)
 		}
 
 		$temp9 = strstr ($data, "Humidity:");
-		$temp9 = strstr ($temp9, "obsInfo2");
-		$humidity = substr ($temp9, 9, strpos ($temp9, "</TD>") - 9);
+		$temp9 = strstr ($temp9, "obsTextA");
+		$humidity = substr ($temp9, 10, strpos ($temp9, "</td>") - 10);
 
 		$temp10 = strstr ($data, "Visibility:");
-		$temp10 = strstr ($temp10, "obsInfo2");	
-		$visibility = substr ($temp10, 9, strpos ($temp10, "</TD>") - 9);
-		if (strncmp ($visibility, "Unlimited", 8) == 0)
+		$temp10 = strstr ($temp10, "obsTextA");	
+		$visibility = substr ($temp10, 10, strpos ($temp10, "</td") - 10);
+		if (strncmp ($visibility, 'Unlimited', 8) == 0)
 		{
-			$visibility = "Unlimited";
+			$visibility = 'Unlimited';
+		}
+		elseif (strlen ($visibility == 0))
+		{
+			$visibility = 'N/A';	
 		}
 		else
 		{	
-			
 			$pos_mile = strpos ($visibility, "mile");
 			$visibility = substr ($visibility, 0, $pos_mile);
 
@@ -632,25 +781,48 @@ function weather_display ($location, $theid)
 		 	}
 		}
 		$temp11 = strstr ($data, "Pressure");
-		$temp11 = strstr ($temp11, "obsInfo2");
-		$temp11 = substr ($temp11, 9, strpos ($temp11, "</TD>") - 9);
+		$temp11 = strstr ($temp11, "obsTextA");
+		$bar_direction = strstr($temp11, "<IMG SRC");
+		$bar_direction = substr ($bar_direction, 0, strpos ($bar_direction, '</td>') - 0);
+		$bar_icon1 = substr ($bar_direction, 0, strpos ($bar_direction, "\n") - 0);
+	  	$bar_icon2 = strstr ($bar_direction, "WIDTH");
+		$bar_icon2 = strstr ($bar_icon2, "http://");
+	
+		if ($weather_proxy_icons == TRUE)
+                { 
+			$bar_icon1 = strstr($bar_icon1, "http://");
+			$bar_icon1 = substr($bar_icon1, 0, strpos ($bar_icon1, '" WIDTH'));
+			if ($weather_proxy_icons_method != "TRANSPARENT")
+                        {	
+				$bar_icon1 = preg_replace("/gif/", "png", $bar_icon1);
+                        }
+                        $bar_icon1 = '<IMG SRC=' . SM_PATH . 'plugins/twc_weather/twc_image_handler.php?wicon=' . $bar_icon1 . '>'; 
+                	
+			$bar_icon2 = strstr($bar_icon2, "http://");
+			$bar_icon2 = substr ($bar_icon2, 0, -2);
+                        if ($weather_proxy_icons_method != "TRANSPARENT")
+                        {
+                                $bar_icon2 = preg_replace("/gif/", "png", $bar_icon2);
+                        }
+                        $bar_icon2 = '<IMG SRC=' . SM_PATH . 'plugins/twc_weather/twc_image_handler.php?wicon=' . $bar_icon2 . '>';
+	
+			$bar_direction = $bar_icon1 . $bar_icon2;
+		}
 
-		$pos_inches = strpos ($temp11, "inches");
-		$pos_direction = strpos ($temp11, "and");
+		$barometer = substr ($temp11, 10, strpos ($temp11, "&nbsp") - 10);
                 
-		$temp_direction = substr ($temp11, $pos_direction);
-
-		$barometer = substr ($temp11, 0, $pos_inches);
+		$pressure_units = $location['pressure-units'];
 		
 		if ($location['use-metric'])
 		{
-			$barometer = SI_Pressure ($barometer) . " KPa";
+			$barometer = SI_Pressure ($barometer, $pressure_units) . " $pressure_units";
 		}
 		else
 		{
 			$barometer = $barometer . " in";
 		}
-		$barometer .= " $temp_direction";
+		$barometer .= " $bar_direction";
+
 		if (strncmp ($feelslike, "N/A", 3) == 0)
 		{
 			$feelslike = $temperature;
@@ -660,7 +832,7 @@ function weather_display ($location, $theid)
 		
                 if ($location['use-metric'])
                 {
-                        $narrative = SI_Narrative($narrative);
+                        $narrative = SI_Narrative($narrative, $location['wind-units']);
                 }
 		else
 		{
@@ -675,16 +847,22 @@ function weather_display ($location, $theid)
 		{
 			global $weather_debug_dir;
 			$fp = fopen ($weather_debug_dir . '/twc_weather-vars.log', 'w');
-                	fwrite ($fp, "City: $citytitle\n");
+			fwrite ($fp, "Server: $the_server\n");
+			fwrite ($fp, "Port: $the_port\n");
+                	fwrite ($fp, "Request: $request\n");
+			fwrite ($fp, "City: $citytitle\n");
 			fwrite ($fp, "Icon: $icon\n");
                 	fwrite ($fp, "Temperature: $temperature\n");
                 	fwrite ($fp, "Conditions: $conditions\n");
                 	fwrite ($fp, "Feels Like: $feelslike\n");
                 	fwrite ($fp, "UV: $uv\n");
                 	fwrite ($fp, "Wind: $wind\n");
+			fwrite ($fp, "Wind Units: " . $location['wind-units'] . "\n");
                 	fwrite ($fp, "Dew: $dew\n");
                 	fwrite ($fp, "Visibility: $visibility\n");
+			fwrite ($fp, "Humidity: $humidity\n");
                 	fwrite ($fp, "Barometer: $barometer\n");
+                	fwrite ($fp, "Pressure Units: '" . $location['pressure-units'] . "'\n");
 			fwrite ($fp, "Narrative: $narrative\n");
 			fwrite ($fp, "Use Metric: " . $location['use-metric'] . "\n");
 			fwrite ($fp, "Narrow: " . $location['forecast-narrow'] . "\n");
@@ -692,11 +870,12 @@ function weather_display ($location, $theid)
 			fwrite ($fp, "Min Cache: " . $weather_cache_time_minimum . "\n");
 			fwrite ($fp, "Max Locales: " . $weather_location_amount . "\n");	                
 			fwrite ($fp, "Timeout: " . $weather_datafeed_timeout . "\n");
-			fwrite ($fp, "Debug: " . $debug . "\n");
+			fwrite ($fp, "Debug: " . $debug . "\n\n");
 			fclose ($fp);
 
-			$fp = fopen ($weather_debug_dir. '/twc_data.log', 'w');
-                	fwrite ($fp, $data);
+			$fp = fopen ($weather_debug_dir. '/twc_weather-data.log', 'w');
+			fwrite ($fp, "Pre-Chunk: \n" . $prechunked_data. "\n\n");
+                	fwrite ($fp, "Post-Chunk: \n" . $data . "\n\n");
                 	fclose ($fp);  
 
 		}
@@ -745,7 +924,9 @@ function weather_display ($location, $theid)
 			$narrative."\\".
 			$location['narrative-days']."\\".
 			$location['use-metric']."\\".
-			$location['forecast-narrow']);
+			$location['forecast-narrow']."\\".
+			$location['wind-units']."\\".
+			$location['pressure-units']);
 	}
 	else
 	{
@@ -759,6 +940,7 @@ function weather_display ($location, $theid)
 		$humidity = $location['cache-humidity'];
 		$visibility = $location['cache-visibility'];
 		$barometer = $location['cache-barometer'];
+		$pressure_units = $location['pressure-units'];
 		$icon = $location['icon'];
 		$narrative = $location['narrative'];
 	}
@@ -860,63 +1042,68 @@ function weather_display ($location, $theid)
 	print "<table style=\"margin-left: " . $table_margin_left . "px; margin-right: " . $table_margin_right . "px; margin-top: " . $table_margin_top . "px; margin-bottom: " . $table_margin_bottom . "px;\" cellpadding=1 cellspacing=0 border=0 bgcolor=" . $table_color_background . " width=" . $table_width . ">";
 	print "<tr><td><table cellpadding=0 cellspacing=0 border=0  width=" . $table_width2 . " bgcolor=" . $table_color_foreground . ">";
 
-         print "<tr><td align=left width=20% style=\"padding-left: 6px\">";
+        print "<tr>";
+        $colspan = '';
  	if ($location['view-icon'])
 	{
+	        print "<td align=left width=\"20%\" style=\"padding-left: 6px\">";
 		print $icon;
+ 		print "</td>";
+		$align = "left";
  	}
 	else
 	{
- 		print "</td>";
+		$align = "center";
+		$colspan = "colspan=2";
 	}
 	if ($location['view-place'])
-                print "<td align=left width=80%><b><font face=\"helvetica\" style=\"font-size:10pt;\">" . $citytitle . "</font></b></td></tr>";
+                print "<td align=" . $align . " " . $colspan . " width=\"80%\"><b><font face=\"helvetica\" style=\"font-size:10pt;\" color=" . $table_color_label . ">" . $citytitle . "</font></b></td></tr>";
 
         if ($location['view-alias'])
-                print "<td align=left><b><font face=\"helvetica\" style=\"font-size: 10pt;\">" . $location['alias'] . "</font></b></td></tr>";
+                print "<td align=". $align ." ". $colspan . "><b><font face=\"helvetica\" style=\"font-size: 10pt;\" color=" . $table_color_label . ">" . $location['alias'] . "</font></b><br></td></tr>";
 
 	if ($location['view-temp'])        
-	print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Temperature:</font></td><td align=left style=\"padding-left: 0px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $temperature . "</font></td></tr>";
+	print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Temperature:") . "</font></td><td align=left style=\"padding-left: 0px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $temperature . "</font></td></tr>";
 		
 	if ($location['view-feels'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Feels Like:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $feelslike . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Feels Like:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $feelslike . "</font></td></tr>";
 
 	if ($location['view-conditions'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Conditions:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $conditions . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Conditions:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $conditions . "</font></td></tr>";
 
 	if ($location['view-uv'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">UV Index:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $uv . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("UV Index:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $uv . "</font></td></tr>";
 
 	if ($location['view-wind'])
-		print "<tr><td align=left valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Wind:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $wind . "</font></td></tr>";
+		print "<tr><td align=left valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Wind:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $wind . "</font></td></tr>";
 
 	if ($location['view-dew'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Dew Point:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $dew . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Dew Point:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $dew . "</font></td></tr>";
 
 	if ($location['view-humidity'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Humidity:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $humidity . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Humidity:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $humidity . "</font></td></tr>";
 
 	if ($location['view-visibility'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Visibility:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $visibility . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Visibility:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $visibility . "</font></td></tr>";
 
 	if ($location['view-barometer'])
-		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Barometer:</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $barometer . "</font></td></tr>";
+		print "<tr><td style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Barometer:") . "</font></td><td><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_value . ">" . $barometer . "</font></td></tr>";
 
 	if ($location['view-narrative'])
 	{
 		if ($location['forecast-narrow'])
 		{
-			print "<tr><td valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Forecast:</font></td><td align=left valign=top><font face=\"helvetica\" style=\"font-size: 8pt;\" color=";
+			print "<tr><td valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Forecast:") . "</font></td><td align=left valign=top><font face=\"helvetica\" style=\"font-size: 8pt;\" color=";
 		}
 		else
 		{
-                	print "<tr><td valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">Forecast:</font></td><td>&nbsp;</td></tr><tr><td align=left valign=top colspan=2 style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=";
+                	print "<tr><td valign=top style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=" . $table_color_label . ">" . _("Forecast:") . "</font></td><td>&nbsp;</td></tr><tr><td align=left valign=top colspan=2 style=\"padding-left: 6px\"><font face=\"helvetica\" style=\"font-size: 8pt;\" color=";
 		}
 
 	print $table_color_value . ">" . $narrative . "</font></td></tr>";
 	}	
 
-	print "<tr><td colspan=2 style=\"padding-top: 6px; padding-left: 6px; padding-right: 6px; text-align: center\" align=left><a href=\"http://w3.weather.com"; 
+	print "<tr><td colspan=2 style=\"padding-top: 6px; padding-left: 6px; padding-right: 6px; text-align: center\" align=left><a href=\"http://www.weather.com"; 
 
 	if (strpos ($location['place'], "US") === false)
 	{
